@@ -2,7 +2,7 @@
  * File: chat.c
  * Author: Justin Zaun
  * Project: GGZ GTK Client
- * $Id: chat.c,v 1.1 2006/10/14 10:27:32 andreasr Exp $
+ * $Id: chat.c 5930 2007-02-10 10:59:26Z andreasr $
  *
  * This file contains all functions that are chat related.
  *
@@ -47,6 +47,11 @@
 #  include <winsock.h>
 #endif
 
+/* Supposedly not present on some systems (solaris). */
+#ifndef INADDR_NONE
+#  define INADDR_NONE ((unsigned long) -1)
+#endif
+
 #include <glib.h>
 
 #include "chat.h"
