@@ -35,12 +35,12 @@ class Profile:
     """
     name       = ''
     protocol   = ''
-    executable = ''
     path       = ''
     arguments  = None
     profiles   = None
     
     def __init__(self):
+        self.executables = []
         self.arguments = []
         self.profiles = {}
 
@@ -53,10 +53,11 @@ class Profile:
             path = []
 
         for directory in path:
-            b = directory + os.sep + self.executable
-            if os.path.isfile(b):
-                self.path = b
-                return
+            for executable in self.executables:
+                b = directory + os.sep + executable
+                if os.path.isfile(b):
+                    self.path = b
+                    return
         self.path = None
         
 def _getXMLText(node):
@@ -131,12 +132,14 @@ def loadProfiles():
             n = p.getElementsByTagName('name')
             assert(len(n) > 0)
             name = _getXMLText(n[0])
-            
+
+            executables = []
             n = p.getElementsByTagName('binary')
             assert(len(n) > 0)
-            executable = _getXMLText(n[0])
-            
-            arguments = [executable]
+            for x in n:
+                executables.append(_getXMLText(x))
+
+            arguments = []
             for x in p.getElementsByTagName('argument'):
                 arguments.append(_getXMLText(x))
 
@@ -147,11 +150,11 @@ def loadProfiles():
                     levels[level.name] = level
 
             profile = Profile()
-            profile.name       = name
-            profile.protocol   = protocol
-            profile.executable = executable
-            profile.arguments  = arguments
-            profile.levels     = levels
+            profile.name        = name
+            profile.protocol    = protocol
+            profile.executables = executables
+            profile.arguments   = arguments
+            profile.levels      = levels
             profiles.append(profile)
     
     return profiles
@@ -174,8 +177,12 @@ class CECPConnection(cecp.Connection):
 
     def onMove(self, move):
         """Called by cecp.Connection"""
-        self.player.moving = True
-        self.player.move(move)
+        if self.player.isReadyToMove():
+            self.player.moving = True
+            self.player.move(move)
+        else:
+            assert(self.player.suppliedMove is None)
+            self.player.suppliedMove = move
         
     def logText(self, text, style):
         """Called by cecp.Connection"""
@@ -218,7 +225,9 @@ class Player(game.ChessPlayer):
     
     __connection = None
     
-    moving = False
+    moving       = False
+    
+    suppliedMove = None
     
     def __init__(self, name, profile, level = 'normal'):
         """Constructor for an AI player.
@@ -242,14 +251,12 @@ class Player(game.ChessPlayer):
         
         # Catch if the child dies
         def cDied(sig, stackFrame):
-            try:
-                os.waitpid(-1, os.WNOHANG)
-            except OSError:
-                pass
+            os.wait()
         signal.signal(signal.SIGCHLD, cDied)
 
         # Fork off a child process to manage the engine
-        if os.fork() == 0:
+        self.__pid = os.fork()
+        if self.__pid == 0:
             # ..
             os.close(toManagerPipe[1])
             os.close(fromManagerPipe[0])
@@ -279,10 +286,7 @@ class Player(game.ChessPlayer):
                 
             # Catch if the child dies
             def childDied(sig, stackFrame):
-                try:
-                    os.waitpid(-1, os.WNOHANG)
-                except OSError:
-                    return
+                os.wait()
                 
                 # Close connection to the application
                 os.close(fromManagerPipe[1])
@@ -302,7 +306,7 @@ class Player(game.ChessPlayer):
                     
                     # One of the connections has closed - kill the engine and quit
                     if len(data) == 0:
-                        os.kill(engineFd, signal.SIGQUIT)
+                        os.kill(self.__pid, signal.SIGQUIT)
                         os._exit(0)
                     
                     # Send data from the application to the engines stdin
@@ -373,6 +377,7 @@ class Player(game.ChessPlayer):
                 return False
             if len(data) == 0:
                 print 'Engine has died'
+                self.die()
                 return False
             self.connection.registerIncomingData(data)
 
@@ -405,8 +410,22 @@ class Player(game.ChessPlayer):
 
     def readyToMove(self):
         """Called by game.ChessPlayer"""
-        self.connection.requestMove()
+        game = self.getGame()
+        whiteTime = game.getWhite().getRemainingTime()
+        blackTime = game.getBlack().getRemainingTime()
+        if game.getWhite() is self:
+            ownTime = whiteTime
+        else:
+            ownTime = blackTime
         
-    def onGameEnded(self, winningPlayer = None):
+        if self.suppliedMove is None:
+            self.connection.requestMove(whiteTime, blackTime, ownTime)
+        else:
+            self.moving = True
+            move = self.suppliedMove
+            self.suppliedMove = None
+            self.move(move)
+        
+    def onGameEnded(self, game):
         """Called by game.ChessPlayer"""
         self.quit()
