@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 
 import os, os.path
-import errno
 import gettext
 
 APP_DATA_DIR    = os.path.join('/usr/local', 'share') 
@@ -23,7 +22,7 @@ import gtk.glade
 gtk.glade.bindtextdomain (DOMAIN, LOCALEDIR)
 gtk.glade.textdomain (DOMAIN)
 
-VERSION = "2.18.2"
+VERSION = "2.19.1"
 APPNAME = _("glChess")
 
 COPYRIGHT     = _('Copyright 2005-2007 Robert Ancell (and contributors)')
@@ -60,8 +59,4 @@ _("You should have received a copy of the GNU General Public License "
 #2D Models:
 #Copyright World Wide Web Consortium, (Massachusetts Institute of Technology, Institut National de Recherche en Informatique et en Automatique, Keio University). All Rights Reserved. http://www.w3.org/Consortium/Legal/"""
 
-try:
-    os.makedirs(DATA_DIR)
-except OSError, e:
-    if e.errno != errno.EEXIST:
-       print 'Unable to make data directory %(dir)s: %(error)s' % {'dir': DATA_DIR, 'error': e.strerror}
+if not os.path.exists(DATA_DIR): os.makedirs(DATA_DIR)
