@@ -16,8 +16,9 @@ import gtk.glade
 gtk.glade.bindtextdomain (DOMAIN, LOCALEDIR)
 gtk.glade.textdomain (DOMAIN)
 
-VERSION = "2.18.1.1"
+VERSION = "2.19.1"
 APPNAME = _("GNOME Sudoku")
+APPNAME_SHORT = _("Sudoku")
 COPYRIGHT = 'Copyright \xc2\xa9 2005-2007, Thomas M. Hinkle'
 DESCRIPTION = _('GNOME Sudoku is a simple sudoku generator and player.  Sudoku is a japanese logic puzzle.')
 AUTHORS = ["Thomas M. Hinkle"]
